# -*- encoding: utf-8 -*-
import marshal
import os

from InquirerPy import prompt

TOKENS = ["github", "youtrack", "drone"]


class Tokens:
    def __init__(self, file_name=os.path.join(os.environ["HOME"], ".suite_py/tokens")):
        self._file_name = file_name
        self._tokens = {}
        try:
            self.load()
        except Exception:
            pass

        self._changed = False
        self.check()
        if self._changed:
            self.save()

    def load(self):
        with open(self._file_name, "rb") as configfile:
            self._tokens = marshal.load(configfile)

    def check(self):
        for token in TOKENS:
            if not self._tokens.get(token):
                self._tokens[token] = prompt(
                    [
                        {
                            "type": "input",
                            "name": token,
                            "message": f"Insert your {token.capitalize()} token:",
                        }
                    ]
                )[token]
                self._changed = True

    def save(self):
        with open(self._file_name, "wb") as configfile:
            marshal.dump(self._tokens, configfile)

    def edit(self, service, token):
        self._tokens[service] = token

    @property
    def github(self):
        return self._tokens["github"]

    @property
    def youtrack(self):
        return self._tokens["youtrack"]

    @property
    def drone(self):
        return self._tokens["drone"]



#! -*- encoding: utf-8 -*-
import os
import sys
import logging

import click
from autoupgrade import Package

from suite_py.__version__ import __version__
from suite_py.commands.ask_review import AskReview
from suite_py.commands.check import Check
from suite_py.commands.create_branch import CreateBranch
from suite_py.commands.release import Release
from suite_py.commands.deploy import Deploy
from suite_py.commands.generator import Generator
from suite_py.commands.id import ID
from suite_py.commands.ip import IP
from suite_py.commands.merge_pr import MergePR
from suite_py.commands.open_pr import OpenPR
from suite_py.commands.project_lock import ProjectLock
from suite_py.commands.status import Status
from suite_py.commands.aggregator import Aggregator
from suite_py.commands.secret import Secret
from suite_py.commands.login import Login
from suite_py.commands.qa import QA
from suite_py.lib.config import Config
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import prompt_utils
from suite_py.lib.tokens import Tokens

from suite_py.lib import logger

ALLOW_NO_GIT_SUBCOMMAND = ["login", "qa", "aggregator"]
ALLOW_NO_HOME_SUBCOMMAND = ["login", "qa", "aggregator"]


@click.group()
@click.option(
    "--project",
    type=click.Path(exists=True),
    default=os.getcwd(),
    help="Path of the project to run the command on (the default is current directory)",
)
@click.option(
    "--timeout",
    type=click.INT,
    help="Timeout in seconds for Captainhook operations",
)
@click.option("-v", "--verbose", count=True)
@click.pass_context
def main(ctx, project, timeout, verbose):
    Package("suite_py").upgrade()
    print(f"v{__version__}")

    config = Config()

    if ctx.invoked_subcommand not in ALLOW_NO_GIT_SUBCOMMAND and not git.is_repo(
        project
    ):
        print(f"the folder {project} is not a git repo")
        sys.exit(-1)

    if ctx.invoked_subcommand not in ALLOW_NO_HOME_SUBCOMMAND and not os.path.basename(
        project
    ) in os.listdir(config.user["projects_home"]):
        print(f"the folder {project} is not in {config.user['projects_home']}")
        sys.exit(-1)

    skip_confirmation = False
    if type(config.user.get("skip_confirmation")).__name__ == "bool":
        skip_confirmation = config.user.get("skip_confirmation")
    elif type(
        config.user.get("skip_confirmation")
    ).__name__ == "list" and ctx.invoked_subcommand in config.user.get(
        "skip_confirmation"
    ):
        skip_confirmation = True

    if not skip_confirmation and not prompt_utils.ask_confirm(
        f"Do you want to continue on project {os.path.basename(project)}?"
    ):
        sys.exit()

    if verbose:
        logger.setLevel(logging.DEBUG)
        logger.debug("Logging as 'DEBUG'")

    ctx.ensure_object(dict)
    ctx.obj["project"] = os.path.basename(project)
    if timeout:
        config.user["captainhook_timeout"] = timeout
    ctx.obj["config"] = config
    ctx.obj["tokens"] = Tokens()

    # Skip chdir if not needed
    if (
        ctx.invoked_subcommand not in ALLOW_NO_GIT_SUBCOMMAND
        or ctx.invoked_subcommand not in ALLOW_NO_HOME_SUBCOMMAND
    ):
        os.chdir(os.path.join(config.user["projects_home"], ctx.obj["project"]))


@main.command(
    "create-branch", help="Create local branch and set the YouTrack card in progress"
)
@click.option("--card", type=click.STRING, help="YouTrack card number (ex. PRIMA-123)")
@click.pass_obj
def cli_create_branch(obj, card):
    CreateBranch(obj["project"], card, obj["config"], obj["tokens"]).run()


@main.command("lock", help="Lock project on staging or prod")
@click.argument(
    "environment", type=click.Choice(("staging", "production", "deploy", "merge"))
)
@click.pass_obj
def cli_lock_project(obj, environment):
    ProjectLock(obj["project"], environment, "lock", obj["config"]).run()


@main.command("unlock", help="Unlock project on staging or prod")
@click.argument(
    "environment", type=click.Choice(("staging", "production", "deploy", "merge"))
)
@click.pass_obj
def cli_unlock_project(obj, environment):
    ProjectLock(obj["project"], environment, "unlock", obj["config"]).run()


@main.command("open-pr", help="Open a PR on GitHub")
@click.pass_obj
def cli_open_pr(obj):
    OpenPR(obj["project"], obj["config"], obj["tokens"]).run()


@main.command("ask-review", help="Requests a PR review")
@click.pass_obj
def cli_ask_review(obj):
    AskReview(obj["project"], obj["config"], obj["tokens"]).run()


@main.command(
    "merge-pr", help="Merge the selected branch to master if all checks are OK"
)
@click.pass_obj
def cli_merge_pr(obj):
    MergePR(obj["project"], obj["config"], obj["tokens"]).run()


@main.group("release", help="Manage releases")
def release():
    pass


@release.command("create", help="Create a github release")
@click.option("--deploy", is_flag=True, help="Trigger deploy after release creation")
@click.pass_obj
def cli_release_create(obj, deploy):
    Release(
        "create", obj["project"], obj["config"], obj["tokens"], flags={"deploy": deploy}
    ).run()


@release.command("deploy", help="Deploy a github release")
@click.pass_obj
def cli_release_deploy(obj):
    Release("deploy", obj["project"], obj["config"], obj["tokens"]).run()


@release.command("rollback", help="Rollback a deployment")
@click.pass_obj
def cli_release_rollback(obj):
    Release("rollback", obj["project"], obj["config"], obj["tokens"]).run()


@main.command("deploy", help="Deploy master branch in production")
@click.pass_obj
def cli_deploy(obj):
    Deploy(obj["project"], obj["config"], obj["tokens"]).run()


@main.command("status", help="Current status of a project")
@click.pass_obj
def cli_status(obj):
    Status(obj["project"], obj["config"]).run()


@main.command("check", help="Verify authorisations for third party services")
@click.pass_obj
def cli_check(obj):
    Check(obj["config"], obj["tokens"]).run()


@main.command("id", help="Get the ID of the hosts where the task is running")
@click.argument("environment", type=click.Choice(("staging", "production")))
@click.pass_obj
def cli_id(obj, environment):
    ID(obj["project"], obj["config"], environment).run()


@main.command("ip", help="Get the IP addresses of the hosts where the task is running")
@click.argument("environment", type=click.Choice(("staging", "production")))
@click.pass_obj
def cli_ip(obj, environment):
    IP(obj["project"], obj["config"], environment).run()


@main.command("generator", help="Generate different files from templates")
@click.pass_obj
def cli_generator(obj):
    Generator(obj["project"], obj["config"], obj["tokens"]).run()


@main.group(
    "aggregator",
    help="Manage CNAMEs of aggregators in QA envs",
    invoke_without_command=True,
)
@click.option(
    "-l", "--list", "show_list", help="deprecated", required=False, count=True
)
@click.option("-c", "--change", "change", help="deprecated", required=False, count=True)
@click.pass_context
def aggregator(ctx, show_list, change):
    if show_list == 1 or change == 1:
        logger.error(
            "suite-py aggregator [-c|-l] has been hard-deprecated. Use suite-py aggregator [list|change] instead."
        )
        sys.exit(0)

    if ctx.invoked_subcommand is None:
        logger.error("Missing command. Try suite-py aggregator --help for help.")
        sys.exit(0)


@aggregator.command("list", help="List all aggregators with the current record")
@click.pass_obj
def cli_aggregator_list(obj):
    Aggregator(obj["config"], "list").run()


@aggregator.command("change", help="Change aggregator record")
@click.pass_obj
def cli_aggregator_change(obj):
    Aggregator(obj["config"], "change").run()


@main.command("login", help="manage login against Auth0")
@click.pass_obj
def login(obj):
    Login(obj["config"]).run()


@main.group("qa", help="Manage QA envs")
def qa():
    pass


@qa.command("update-quota", help="Update quota in QA for a user")
@click.pass_obj
def cli_qa_update_quota(obj):
    QA("update-quota", obj["project"], obj["config"], obj["tokens"]).run()


@qa.command("list", help="List QA envs for user: all to show qa of all users.")
@click.option("-u", "--user", "user", required=False)
@click.option("-s", "--status", "status", multiple=True, type=str)
@click.option("-c", "--card", "card", type=str)
@click.pass_obj
def cli_qa_list(obj, user, status, card):
    QA(
        "list",
        obj["project"],
        obj["config"],
        obj["tokens"],
        {"user": user, "status": status, "card": card},
    ).run()


@qa.command("create", help="Create QA env")
@click.argument("microservices", nargs=-1, required=True)
@click.pass_obj
def cli_qa_create(obj, microservices):
    QA(
        "create",
        obj["project"],
        obj["config"],
        obj["tokens"],
        {"services": microservices},
    ).run()


@qa.command("update", help="Update QA env")
@click.argument("qa_hash", required=True)
@click.argument("microservices", nargs=-1, required=True)
@click.pass_obj
def cli_qa_update(obj, qa_hash, microservices):
    QA(
        "update",
        obj["project"],
        obj["config"],
        obj["tokens"],
        {"hash": qa_hash, "services": microservices},
    ).run()


@qa.command("delete", help="Delete QA env")
@click.argument("qa_hash", required=True)
@click.pass_obj
def cli_qa_delete(obj, qa_hash):
    QA("delete", obj["project"], obj["config"], obj["tokens"], {"hash": qa_hash}).run()


@qa.command("freeze", help="Freeze QA env")
@click.argument("qa_hash", required=True)
@click.pass_obj
def cli_qa_freeze(obj, qa_hash):
    QA("freeze", obj["project"], obj["config"], obj["tokens"], {"hash": qa_hash}).run()


@qa.command("unfreeze", help="Unfreeze QA env")
@click.argument("qa_hash", required=True)
@click.pass_obj
def cli_qa_unfreeze(obj, qa_hash):
    QA(
        "unfreeze", obj["project"], obj["config"], obj["tokens"], {"hash": qa_hash}
    ).run()


@qa.command("check", help="Check QA conf")
@click.pass_obj
def cli_qa_check(obj):
    QA("check", obj["project"], obj["config"], obj["tokens"]).run()


@qa.command("describe", help="Describe QA environment")
@click.argument("qa_hash", required=True)
@click.option("--json", is_flag=True, default=False, help="Get response as JSON")
@click.pass_obj
def cli_qa_describe(obj, qa_hash, json):
    flags = {"hash": qa_hash, "json": json}
    QA("describe", obj["project"], obj["config"], obj["tokens"], flags).run()


@qa.command(
    "toggle-maintenance",
    help="Toggle maintenance mode (requires 'manage:maintenance' permission)",
)
@click.pass_obj
def cli_qa_toggle_maintenance(obj):
    QA("toggle-maintenance", obj["project"], obj["config"], obj["tokens"]).run()


@main.group(
    "secret", help="Manage secrets grants in multiple countries (aws-vault needed)"
)
def secret():
    pass


@secret.command("create", help="Create a new secret")
@click.option("-b", "--base-profile", "base_profile", required=False)
@click.pass_obj
def cli_secret_create(obj, base_profile):
    Secret(obj["project"], obj["config"], "create", base_profile).run()


@secret.command("grant", help="Grant permissions to an existing secret")
@click.option("-b", "--base-profile", "base_profile", required=False)
@click.pass_obj
def cli_secret_grant(obj, base_profile):
    Secret(obj["project"], obj["config"], "grant", base_profile).run()



# -*- coding: utf-8 -*-
import sys
import re
import semver

from halo import Halo

from suite_py.lib import logger
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import prompt_utils
from suite_py.lib.handler.captainhook_handler import CaptainHook
from suite_py.lib.handler.drone_handler import DroneHandler
from suite_py.lib.handler.git_handler import GitHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.commands.release import _parse_available_countries


class Deploy:
    # pylint: disable=too-many-instance-attributes
    def __init__(self, project, config, tokens):
        self._project = project
        self._config = config
        self._youtrack = YoutrackHandler(config, tokens)
        self._captainhook = CaptainHook(config)
        self._github = GithubHandler(tokens)
        self._repo = self._github.get_repo(project)
        self._git = GitHandler(project, config)
        self._drone = DroneHandler(config, tokens, repo=project)
        self._countries = _parse_available_countries(self._drone)

    def run(self):
        self._stop_if_prod_locked()

        self._git.fetch()

        if len(self._countries) > 0:
            logger.error(
                "Deploy command cannot be used on this project. Try to run `suite-py release` instead."
            )
            sys.exit(1)

        tags = self._repo.get_tags()
        tag = git.get_last_tag_number(tags)
        latest_release = self._get_release(tag)

        current_version = latest_release.tag_name if latest_release else tag

        if current_version:
            logger.info(f"The current release is {current_version}")
            versions = _bump_versions(current_version)
            commits = self._github.get_commits_since_release(
                self._repo, current_version
            )

            _check_migrations_deploy(commits)

            message = "\n".join(
                ["* " + c.commit.message.splitlines()[0] for c in commits]
            )

            logger.info(f"\nCommits list:\n{message}\n")

            if not prompt_utils.ask_confirm("Do you want to continue?"):
                sys.exit()

            new_version = prompt_utils.ask_choices(
                "Select version:",
                [
                    {"name": f"Patch {versions['patch']}", "value": versions["patch"]},
                    {"name": f"Minor {versions['minor']}", "value": versions["minor"]},
                    {"name": f"Major {versions['major']}", "value": versions["major"]},
                ],
            )

        else:
            # Se non viene trovata la release e non ci sono tag, viene saltato il check delle migrations e l'update delle card su youtrack
            logger.warning("No tags found, I'm about to push the tag 0.1.0")
            if not prompt_utils.ask_confirm(
                "Are you sure you want to continue?", default=False
            ):
                sys.exit()
            new_version = "0.1.0"
            message = f"First release with tag {new_version}"

        self._create_release(new_version, message)
        self._manage_youtrack_card(commits, new_version)

    def _stop_if_prod_locked(self):
        request = self._captainhook.status(self._project, "production")
        if request.status_code != 200:
            logger.error("Unable to determine lock status on master.")
            sys.exit(-1)

        request_object = request.json()
        if request_object["locked"]:
            logger.error(
                f"The project is locked in production by {request_object['by']}. Unable to continue."
            )
            sys.exit(-1)

    def _get_release(self, tag):
        with Halo(text="Loading...", spinner="dots", color="magenta"):
            latest_release = self._github.get_latest_release_if_exists(self._repo)
            if latest_release and latest_release.title == tag:
                return latest_release
        return None

    def _create_release(self, new_version, message):
        new_release = self._repo.create_git_release(
            new_version,
            new_version,
            self._youtrack.replace_card_names_with_md_links(message),
        )
        if new_release:
            logger.info(f"The release has been created! Link: {new_release.html_url}")

            build_number = self._drone.get_build_from_tag(new_version)
            if build_number:
                drone_url = self._drone.get_build_url(build_number)
                logger.info(
                    f"You can follow the deployment in production here: {drone_url}"
                )

    def _manage_youtrack_card(self, commits, new_version):
        release_state = self._config.youtrack["release_state"]

        issue_ids = self._youtrack.get_issue_ids(commits)

        if len(issue_ids) > 0:
            update_youtrack_state = prompt_utils.ask_confirm(
                f"Do you want to move the associated cards to {release_state} state?",
                default=False,
            )

            for issue_id in issue_ids:
                try:
                    self._youtrack.comment(
                        issue_id,
                        f"Deploy in production of {self._project} done with the release {new_version}",
                    )
                    if update_youtrack_state:
                        self._youtrack.update_state(issue_id, release_state)
                        logger.info(f"{issue_id} moved to {release_state}")
                except Exception:
                    logger.warning(
                        f"An error occurred while moving the card {issue_id} to {release_state}"
                    )
                repos_status = self._get_repos_status_from_issue(issue_id)
                if all(r["deployed"] for r in repos_status.values()):
                    try:
                        self._youtrack.update_deployed_field(issue_id)
                        logger.info("Custom field Deployed updated on YouTrack")
                    except Exception:
                        logger.warning(
                            "An error occurred while updating the custom field Deployed"
                        )

    def _get_repos_status_from_issue(self, issue_id):
        regex_pr = r"^PR .* -> https:\/\/github\.com\/primait\/(.*)\/pull\/([0-9]*)$"
        regex_deploy = r"^Deploy in production of (.*) done with the release"
        comments = self._youtrack.get_comments(issue_id)
        repos_status = {}

        for c in comments:
            m = re.match(regex_pr, c["text"])
            if m:
                project = m.group(1)
                pr_number = int(m.group(2))
                repos_status[project] = {}
                repos_status[project]["pr"] = pr_number
                repos_status[project]["deployed"] = False
            m = re.match(regex_deploy, c["text"])
            if m:
                project = m.group(1)
                try:
                    repos_status[project]["deployed"] = True
                except Exception:
                    pass
        return repos_status


def _bump_versions(current):
    return {
        "patch": semver.bump_patch(current),
        "minor": semver.bump_minor(current),
        "major": semver.bump_major(current),
    }


def _check_migrations_deploy(commits):
    if not commits:
        logger.error("ERROR: no commit found")
        sys.exit(-1)
    elif len(commits) == 1:
        files_changed = git.files_changed_between_commits("--raw", f"{commits[0].sha}~")
    else:
        files_changed = git.files_changed_between_commits(
            f"{commits[-1].sha}~", commits[0].sha
        )
    if git.migrations_found(files_changed):
        logger.warning("WARNING: migrations detected in the code")
        if not prompt_utils.ask_confirm(
            "Are you sure you want to continue?", default=False
        ):
            sys.exit()



# -*- encoding: utf-8 -*-
from github import Github


class GithubHandler:
    _organization = "primait"

    def __init__(self, tokens):
        self._client = Github(tokens.github)

    def get_repo(self, repo_name):
        return self._client.get_repo(f"{self._organization}/{repo_name}")

    def get_organization(self):
        return self._client.get_organization(self._organization)

    def get_user(self):
        return self._client.get_user()

    def create_pr(self, repo, branch, title, body="", base="master", is_draft=False):
        return self.get_repo(repo).create_pull(
            title=title, head=branch, base=base, body=body, draft=is_draft
        )

    def get_pr(self, repo, pr_number):
        return self.get_repo(repo).get_pull(pr_number)

    def get_branch_from_pr(self, repo, pr_number):
        repo = self.get_repo(repo)
        return repo.get_branch(repo.get_pull(pr_number).head.ref)

    def get_team_members(self, team_name=""):
        return self.get_organization().get_team_by_slug(team_name).get_members()

    def get_all_users(self):
        return self.get_organization().get_members()

    def get_list_pr(self, repo):
        pulls = self.get_repo(repo).get_pulls(
            state="open", sort="created", base="master"
        )
        return pulls

    def get_pr_from_branch(self, repo, branch):
        return self.get_repo(repo).get_pulls(head=f"primait:{branch}")

    def get_link_from_pr(self, repo, pr_number):
        return f"https://github.com/primait/{repo}/pull/{pr_number}"

    def get_commits_since_release(self, repo, tag):
        release_commit = repo.get_commit(tag)
        commits = []
        for c in repo.get_commits():
            if c == release_commit:
                break
            commits.append(c)
        return commits

    def get_latest_release_if_exists(self, repo):
        try:
            return repo.get_latest_release()
        except Exception:
            return None

    def user_is_admin(self, repo):
        return self.get_repo(repo).permissions.admin

    def get_release_if_exists(self, repo, release):
        try:
            return repo.get_release(release)
        except Exception:
            return None

    def get_build_status(self, repo, ref):
        return self.get_repo(repo).get_commit(ref).get_combined_status()

    def get_releases(self, repo):
        return self.get_repo(repo).get_releases()

    def get_branches(self, repo):
        return self.get_repo(repo).get_branches()

    def get_raw_content(self, repo, ref, file):
        repo = self.get_repo(repo)
        content = repo.get_contents(file, ref)
        return content.decoded_content



# -*- coding: utf-8 -*-
from datetime import datetime
import os
from os import listdir
from os.path import isfile, join
import stat
import subprocess

from jinja2 import Environment, DebugUndefined, FileSystemLoader
import yaml

from suite_py.lib import logger
from suite_py.lib.handler import prompt_utils
from suite_py.lib.handler.drone_handler import DroneHandler
from suite_py.lib.handler.git_handler import GitHandler
from suite_py.lib.handler.github_handler import GithubHandler

NOW = datetime.now().strftime("%Y%m%d%H%M")


class Generator:
    def __init__(self, project, config, tokens):
        self._project = project
        self._config = config
        self._tokens = tokens
        self._artemide_path = os.path.join(config.user["projects_home"], "artemide")
        self._drone_values_path = f"{self._artemide_path}/templates/drone/values/"
        self._deploy_scripts_values_path = (
            f"{self._artemide_path}/templates/scripts/values/"
        )
        self._github = GithubHandler(tokens)

    def run(self):
        logger.warning(
            "This command requires shellcheck and shfmt, please install these tools before continuing"
        )
        logger.warning(
            "see https://github.com/koalaman/shellcheck#installing and https://github.com/mvdan/sh#shfmt for installation"
        )
        # move to artemide root directory
        os.chdir(self._artemide_path)

        env = Environment(
            loader=FileSystemLoader(self._artemide_path),
            undefined=DebugUndefined,
            trim_blocks=True,
            lstrip_blocks=True,
        )
        env.filters["to_yaml"] = to_yaml
        env.add_extension("jinja2.ext.do")

        template_type = prompt_utils.ask_choices(
            "What files do you want to generate?", ["drone", "deploy_scripts"]
        )
        commit_message = f"Autogenerated update {template_type} from suite-py {NOW}"

        autobranch = prompt_utils.ask_confirm(
            "Do you want to create branch and PR automatically?", default=False
        )

        if autobranch:
            autopush = True
        else:
            autopush = prompt_utils.ask_confirm(
                "Do you want to push automatically?", default=False
            )

        choice = prompt_utils.ask_questions_input(
            "What project do you want to work on? Enter `all` to select them all: ",
            self._project,
        )

        if choice == "all":
            projects = self._get_all_templated_projects(template_type)
        else:
            projects = choice.split(" ")

        skipped = []
        for project in projects:
            logger.info(f"{project}: Checking if there are no uncommitted files")
            git = GitHandler(project, self._config)
            git.check_repo_cloned()
            if git.is_dirty():
                if not prompt_utils.ask_confirm(
                    f"{project}: There are changes already present, do you want to continue anyway?",
                    default=False,
                ):
                    logger.error(f"{project}: skipping the project")
                    skipped.append(project)
                    continue

            branch_name = _configure_branch(git, template_type, autobranch, autopush)

            logger.info(f"{project}: Creation in progress")
            # launch generate
            if template_type == "drone":
                self._generate_drone(
                    project, f"{self._drone_values_path}{project}.yml", env
                )
            elif template_type == "deploy_scripts":
                _generate_build_script(
                    git.get_path(),
                    f"{self._deploy_scripts_values_path}{project}.yml",
                    env,
                )
                _generate_deploy_script(
                    git.get_path(),
                    f"{self._deploy_scripts_values_path}{project}.yml",
                    env,
                )
            logger.info(f"{project}: Creation completed")

            if autobranch or autopush:
                # controllo che sia cambiato effettivamente qualcosa
                if git.is_dirty(untracked=True):
                    self._git_operations(
                        git, autobranch, autopush, branch_name, commit_message
                    )
                else:
                    logger.warning(
                        f"{project}: no git operations to do. No files modified"
                    )

            logger.warning(f"Skipped projects: {skipped}")

    def _get_all_templated_projects(self, template_type):
        if template_type == "drone":
            values_path = self._drone_values_path
        elif template_type == "deploy_scripts":
            values_path = self._deploy_scripts_values_path

        return [
            f.replace(".yml", "")
            for f in listdir(values_path)
            if isfile(join(values_path, f))
        ]

    def _git_operations(self, git, autobranch, autopush, branch_name, message):
        if autobranch:
            logger.info(f"{git.get_repo()}: creating branch {branch_name}")
            git.checkout(branch_name, new=True)
            git.add()
            git.commit(message)
            git.push(branch_name)
            pr = self._github.create_pr(git.get_repo(), branch_name, message)
            logger.info(f"Pull request with number {pr.number} created! {pr.html_url}")
        elif autopush:
            if git.current_branch_name() != "master":
                git.add()
                git.commit(message)
                git.push(branch_name)
            else:
                logger.warning(
                    f"{git.get_repo()} is on master, skipping automatic push"
                )

    def _generate_drone(self, project, values_file, env):
        values = yaml.safe_load(open(values_file, encoding="utf-8"))
        template = env.get_template(f"templates/drone/{values['template']}.j2")
        rendered = template.render(values)

        _write_on_repo(
            os.path.join(self._config.user["projects_home"], project),
            rendered,
            ".drone.yml",
        )
        drone = DroneHandler(self._config, self._tokens, repo=project)
        drone.fmt()
        drone.validate()
        drone.sign()


def _configure_branch(git, template_type, autobranch, autopush):
    logger.info(f"{git.get_repo()}: Synchronizing the repo with the remote")
    if autobranch:
        git.sync()  # fa anche checkout su master
        return f"t/autogenerated_update_{template_type}_{NOW}"

    current_branch = git.current_branch_name()
    if autopush and git.remote_branch_exists(current_branch):
        git.fetch()
        git.pull()

    return current_branch


def _generate_build_script(project_path, values_file, env):
    file_name = "build"
    values = yaml.safe_load(open(values_file, encoding="utf-8"))

    val = {**values["info"], **values["build"]}
    template = env.get_template(f"templates/scripts/build/{val['template']}")
    rendered = template.render(val)

    _write_on_repo(os.path.join(project_path, "deploy"), rendered, file_name)
    _format_script(f"{project_path}/deploy/{file_name}")
    _validate_script(f"{project_path}/deploy/{file_name}")


def _generate_deploy_script(project_path, values_file, env):
    values = yaml.safe_load(open(values_file, encoding="utf-8"))

    for country, country_props in values["deploy"].items():
        val = {**values["info"], **country_props}
        template = env.get_template("templates/scripts/deploy/base.j2")
        rendered = template.render(val)
        file_name = f"deploy-{country}"
        _write_on_repo(os.path.join(project_path, "deploy"), rendered, file_name)
        _format_script(f"{project_path}/deploy/{file_name}")
        _validate_script(f"{project_path}/deploy/{file_name}")


def _write_on_repo(path, rendered, file_name):
    if not os.path.exists(path):
        os.mkdir(path, mode=0o755)

    with open(os.path.join(path, file_name), "w", encoding="utf-8") as fd:
        fd.write(rendered)


def _format_script(filepath):
    subprocess.run(["shfmt", "-i", "2", "-l", "-w", filepath], check=True)


def _validate_script(filepath):
    _make_executable(filepath)
    subprocess.run(["shellcheck", filepath], check=True)


def _make_executable(filepath):
    st = os.stat(filepath)
    os.chmod(filepath, st.st_mode | stat.S_IEXEC)


def to_yaml(d, indent=10):
    return yaml.dump(d, indent=indent)



# -*- encoding: utf-8 -*-
import inquirer
from InquirerPy import prompt


def ask_confirm(text, default=True):
    questions = [
        {"type": "confirm", "message": text, "name": "continue", "default": default}
    ]
    answer = prompt(questions)
    return bool(answer["continue"])


def ask_questions_input(question_text, default_text=""):
    questions = [
        {
            "type": "input",
            "name": "value",
            "message": question_text,
            "default": default_text,
        }
    ]

    answer = prompt(questions)
    return answer["value"]


def ask_questions_editor(question_text, default_text=""):
    questions = [
        inquirer.Editor(
            "value",
            message=question_text,
            default=default_text,
        )
    ]

    answer = inquirer.prompt(questions)
    return answer["value"]


def ask_choices(question_text, choices, default_text=""):
    questions = [
        {
            "type": "list",
            "name": "value",
            "message": question_text,
            "choices": choices,
            "default": default_text,
        }
    ]

    answer = prompt(questions)
    return answer["value"]


def ask_multiple_choices(question_text, choices):
    questions = [
        {
            "type": "checkbox",
            "name": "values",
            "message": question_text,
            "choices": [
                {"name": choice, "value": choice, "enabled": False}
                for choice in choices
            ],
        }
    ]

    answer = prompt(questions)
    return answer["values"]



# -*- encoding: utf-8 -*-
import re
import time
from urllib.parse import urljoin

from suite_py.lib.requests.session import Session
from suite_py.lib.requests.auth import BearerAuth

REGEX = r"([A-Za-z]+-[0-9]+)"


class YoutrackHandler:
    def __init__(self, config, tokens):
        headers = {"Accept": "application/json", "Content-Type": "application/json"}
        self._base_url = config.youtrack["url"] + "/"
        self._issue_url = urljoin(self._base_url, "issue/")
        self._client = Session(base_url=urljoin(self._base_url, "api/"))
        self._client.headers = headers
        self._client.auth = BearerAuth(tokens.youtrack)

    def get_projects(self):
        params = {"fields": "id,name,shortName"}
        return self._client.get("admin/projects", params=params).json()

    def get_current_user(self):
        params = {"fields": "login"}
        return self._client.get("users/me", params=params).json()

    def get_issue(self, issue_id):
        params = {
            "fields": "$type,id,idReadable,summary,customFields(name,value(name))"
        }
        issue = self._client.get(f"issues/{issue_id}", params=params).json()
        type_field = [x for x in issue["customFields"] if x["name"] == "Type"][0]
        issue["Type"] = type_field["value"]["name"]
        return issue

    def get_comments(self, issue_id):
        params = {"fields": "$type,id,text"}
        return self._client.get(f"issues/{issue_id}/comments", params=params).json()

    def update_deployed_field(self, issue_id):
        payload = {
            "customFields": [
                {
                    "name": "Deployed",
                    "$type": "SimpleIssueCustomField",
                    "value": time.time() * 1000,
                }
            ]
        }
        self._client.post(f"issues/{issue_id}", json=payload)

    def validate_issue(self, issue_id):
        try:
            if self.get_issue(issue_id):
                return True
        except Exception:
            pass
        return False

    def comment(self, issue_id, comment):
        payload = {"text": comment}
        self._client.post(f"issues/{issue_id}/comments", json=payload)

    def update_state(self, issue_id, status):
        payload = {
            "customFields": [
                {
                    "name": "State",
                    "$type": "StateIssueCustomField",
                    "value": {"name": status},
                }
            ]
        }
        self._client.post(f"issues/{issue_id}", json=payload)

    def add_tag(self, issue_id, label):
        params = {"fields": "id,name", "query": label}
        tag = self._client.get("issueTags", params=params).json()

        params = {"fields": "$type,id,tags($type,id,name)"}
        issue = self._client.get(f"issues/{issue_id}", params=params).json()

        issue["tags"].append(tag[0])

        payload = {"tags": issue["tags"]}
        self._client.post(f"issues/{issue_id}", json=payload)

    def assign_to(self, issue_id, user):
        payload = {
            "customFields": [
                {
                    "name": "Assignee",
                    "$type": "SingleUserIssueCustomField",
                    "value": {"login": user},
                }
            ]
        }
        self._client.post(f"issues/{issue_id}", json=payload)

    def get_link(self, issue_id):
        return f"{self._issue_url}{issue_id}"

    def get_issue_ids(self, commits):
        issue_ids = []
        for c in commits:
            issue_id = self.get_card_from_name(c.commit.message)
            if issue_id:
                issue_ids.append(issue_id)
        return issue_ids

    def get_card_from_name(self, name):
        if re.search(REGEX, name):
            id_card = re.findall(REGEX, name)[0]
            if self.validate_issue(id_card):
                return id_card
        return None

    def get_ids_from_release_body(self, body):
        return list(set(re.findall(REGEX, body)))

    def replace_card_names_with_md_links(self, text):
        return re.sub(REGEX, f"[\\1]({self._issue_url}\\1)", text)
